package de.firstdata.ipg.api.client;

import java.math.BigDecimal;
import java.util.List;

import de.firstdata.ipg.api.client.transaction.IPGApiTransaction;
import de.firstdata.ipg.api.schema.IPGApiOrderResponse;
import de.firstdata.ipg.api.schema.Secure3DResponse;
import de.firstdata.ipg.api.schema.SecurePlusResponse;

/**
 * An <code>IPGApiResult</code> object is returned when successfully calling
 * {@link IPGApiClient#commitTransaction(IPGApiTransaction)}. This class
 * contains <code>getter</code> methods for accessing the data fields which give
 * detailed information about the transaction approval.
 *
 * @author Andreas Schmid
 */
public class IPGApiResult {

	private final IPGApiOrderResponse _response;

	/**
	 * the constructor is invoked internally during the execution of
	 * {@link IPGApiClient#commitTransaction(IPGApiTransaction)}
	 *
	 * @param response
	 *            the unmarshalled response object
	 */
	public IPGApiResult(final IPGApiOrderResponse response) {
		this._response = response;
	}

	/**
	 * returns ExternalMerchantId from Response
	 *
	 * @return ExternalMerchantID
	 */
	public String getExternalMerchantID() {
		return this._response.getExternalMerchantID();
	}

	/**
	 * returns debug information - for internal use only
	 *
	 * @return debug information
	 */
	public String getDebugInformation() {
		return this._response.getDebugInformation();
	}

	/**
	 * returns the commercial service provider
	 *
	 * @return the commercial service provider - currently, <i>BNLP</i> will
	 *         always be returned
	 */
	public String getCommercialServiceProvider() {
		return this._response.getCommercialServiceProvider();
	}

	/**
	 * returns the time stamp set by the First Data API Web Service before
	 * returning the transaction result
	 *
	 * @return the time stamp in millisecs representing the difference between
	 *         the current time and midnight, January 1, 1970 UTC
	 */
	public String getTransactionTime() {
		return this._response.getTransactionTime();
	}

	/**
	 * returns the reference number the credit card processor uses for referring
	 * to this transaction
	 *
	 * @return the reference number as returned by the credit card processor
	 */
	public String getProcessorReferenceNumber() {
		return this._response.getProcessorReferenceNumber();
	}

	/**
	 * returns the processing response message returned by the credit card
	 * processor - since the transaction has been approved, the string
	 * <i>Function performed error-free</i> is always returned
	 *
	 * @return the string <i>Function performed error-free</i>
	 */
	public String getProcessorResponseMessage() {
		return this._response.getProcessorResponseMessage();
	}

	/**
	 * returns the processing error message - note that this element is always
	 * empty since the transaction has been approved
	 *
	 * @return an empty string
	 */
	public String getErrorMessage() {
		return this._response.getErrorMessage();
	}

	/**
	 * returns the order ID which must be provided when referring to this
	 * transaction - you have to provide this ID when creating a follow-up
	 * transaction by calling one of the operations
	 * {@link IPGApiClient#createPostAuthTransaction(double, String)},
	 * {@link IPGApiClient#createReturnTransaction(double, String)}, and
	 * {@link IPGApiClient#createVoidTransaction(String, String)}
	 *
	 * @return the transaction's order ID as returned by the First Data API Web
	 *         Service
	 */
	public String getOrderId() {
		return this._response.getOrderId();
	}

	/**
	 * returns the code which is created by the credit card processor when
	 * having approved this transaction
	 *
	 * @return the approval code as returned by the credit card processor
	 */
	public String getApprovalCode() {
		return this._response.getApprovalCode();
	}

	/**
	 * returns the address verification response (AVS)
	 */
	public String getAVSResponse() {
		return this._response.getAVSResponse();
	}

	/**
	 * returns the tDate which must be provided when voiding this transaction by
	 * creating a <i>Void</i> transaction object via
	 * {@link IPGApiClient#createVoidTransaction(String, String)}
	 *
	 * @return the transaction's tDate as returned by the First Data API Web
	 *         Service
	 */
	public String getTDate() {
		return this._response.getTDate();
	}

	/**
	 * returns the transaction result which is always set to <i>APPROVED</i>
	 *
	 * @return the string <i>APPROVED</i>
	 */
	public String getTransactionResult() {
		return this._response.getTransactionResult();
	}

	/**
	 * returns the terminal id of this transaction
	 */
	public String getTerminalID() {
		return this._response.getTerminalID();
	}

	/**
	 * Returns the processor response code
	 *
	 * @return
	 */
	public String getProcessorResponseCode() {
		return this._response.getProcessorResponseCode();
	}

	/**
	 * Returns the processor approval code
	 *
	 * @return
	 */
	public String getProcessorApprovalCode() {
		return this._response.getProcessorApprovalCode();
	}

	/**
	 * Returns the processor receipt number
	 *
	 * @return
	 */
	public String getProcessorReceiptNumber() {
		return this._response.getProcessorReceiptNumber();
	}

	/**
	 * Returns the processor trace number
	 *
	 * @return
	 */
	public String getProcessorTraceNumber() {
		return this._response.getProcessorTraceNumber();
	}

	public String getProcessorCCVResponse() {
		return this._response.getProcessorCCVResponse();
	}

	public String getBrand() {
		return this._response.getBrand();
	}

	public String getCountry() {
		return this._response.getCountry();
	}

	public String getPayerSecurityLevel() {
		return this._response.getPayerSecurityLevel();
	}

	public String getPaymentType() {
		return this._response.getPaymentType();
	}

	public String getTDateFormatted() {
		return this._response.getTDateFormatted();
	}

	public String getMandateReference() {
		return this._response.getMandateReference();
	}

	public String getRedirectURL() {
		return this._response.getRedirectUrl();
	}

	public BigDecimal getProcessorInstallmentFirstAmount() {
		return this._response.getProcessorInstallmentFirstAmount();
	}

	public BigDecimal getProcessorInstallmentOtherAmount() {
		return this._response.getProcessorInstallmentOtherAmount();
	}

	public BigDecimal getProcessorInstallmentIssuerFeeAmount() {
		return this._response.getProcessorInstallmentIssuerFeeAmount();
	}

	public BigDecimal getProcessorInstallmentTaxesAmount() {
		return this._response.getProcessorInstallmentTaxesAmount();
	}

	public BigDecimal getProcessorInstallmentInsuranceAmount() {
		return this._response.getProcessorInstallmentInsuranceAmount();
	}

	public BigDecimal getProcessorInstallmentOtherExpensesAmount() {
		return this._response.getProcessorInstallmentOtherExpensesAmount();
	}

	public BigDecimal getProcessorInstallmentTotalAmount() {
		return this._response.getProcessorInstallmentTotalAmount();
	}

	public BigDecimal getProcessorInstallmentRatePerYear() {
		return this._response.getProcessorInstallmentRatePerYear();
	}

	public BigDecimal getProcessorVoucherRemainingAmount() {
		return this._response.getProcessorVoucherRemainingAmount();
	}

	public String getProcessorVoucherType() {
		return this._response.getProcessorVoucherType();
	}

	public Long getIpgTransactionId() {
		return this._response.getIpgTransactionId();
	}

	public List<String> getHostedDataIds() {
		return this._response.getHostedData().getHostedDataID();
	}

	public Secure3DResponse getSecure3DResponse() {
		return this._response.getSecure3DResponse();
	}

	public SecurePlusResponse getSecurePlusResponse() {
		return this._response.getSecurePlusResponse();
	}

	public String getProcessorNetworkInformation() {
		return this._response.getProcessorNetworkInformation();
	}

	public String getUpdatedAccountStatus() {
		return this._response.getUpdatedAccountStatus();
	}

	public String getUpdatedPrimaryAccountNumber() {
		return this._response.getUpdatedPrimaryAccountNumber();
	}

	public String getUpdatedExpirationDate() {
		return this._response.getUpdatedExpirationDate();
	}

	public String getUpdatedAccountErrorCode() {
		return this._response.getUpdatedAccountErrorCode();
	}
}
